<?php
/*--------------------------------------------------------------------------------------------------*/
/*** CLASSES
----------------------------------------------------------------------------------------------------*/


/*** Class Booking - Get Classes
--------------------------------------------------------------------------------------------------*/
function cr_get_classes($locationid) {

  $content  = array();
  $current_day = date('l');
  $i = 0;
  $count_limit = 14;

  // Class ID
  $classID = get_field('global_cr_class_id','option');
  // Class Category ID
  $classCatID = get_field('global_cr_class_cat_id','option');

  // Class Days
  $classDays = get_field('global_cr_class_days','option');

  // Build Time filter so as not to show classes that already happened
  if (get_field('state','option')) {
    $current_state = trim(get_field('state','option'));
  }
  else {
    $current_state = 'CA';
  }
  $current_tz = cr_get_timezone($current_state);

  $current_time = new DateTime();
  $current_time->setTimezone(new DateTimeZone($current_tz));

  while ($i < $count_limit) {

    $start_offset = $i*$classDays;
    $offset_string_start = '+'.$start_offset.' day';
    $offset_string_end = '+'.($start_offset+$classDays).' day';

    $start_date = date('Y-n-j', strtotime($offset_string_start));
 
    $end_date = date('Y-n-j', strtotime($offset_string_end));
 
    $api_url = CLUBREADY_API_URL . '/scheduling/class-schedule?ApiKey=' . CLUBREADY_API_KEY . '&StoreID=' . $locationid . '&FromDate=' . $start_date . '&ToDate=' . $end_date . '&format=json&FilterByCategoryId=' . $classCatID;

    $response = cr_booking_get_curl($api_url);
    
    $myAllContent = json_decode($response);

      $matchingClasses = array();
      foreach($myAllContent as $myContent) {

        $classDate = $myContent->Date;
        $classTime = $myContent->StartTime;

        $returnTime = DateTime::createFromFormat('n-j-Y g:i a', $classDate . ' ' .$classTime, new DateTimeZone($current_tz));

        if ( $current_time->getTimestamp() < $returnTime->getTimestamp() ) {

            if( $classID ) {
            if($myContent->ClassId == $classID && $myContent->FreeSpots > 0) {
                //TODO: Make sure that this can handle multiple options -- not just a single one
                array_push($matchingClasses,$myContent);
            }
            } else {
            if($myContent->FreeSpots > 0) {
                //TODO: Make sure that this can handle multiple options -- not just a single one
                array_push($matchingClasses,$myContent);
            }
            }

        }
      }

      $content = array_merge($content,$matchingClasses);

    if (count($content) > 0) {
      break;
    } else {
      $i++;
    }

  } // END WHILE LOOP

  return $content;

}



/*** Class Booking - Build Query String
--------------------------------------------------------------------------------------------------*/
function cr_build_booking_querystring($post) {

  $params = array(
    'ApiKey' => CLUBREADY_API_KEY,
    'UserId' => $post['signup-userID'],
    'StoreId' => $post['signup-clubID'],
    'ScheduleId' => $post['ScheduleId'],
  );

  $query = http_build_query($params);

  return $query;

}

/*** Get timezones
--------------------------------------------------------------------------------------------------*/
function cr_get_timezone($state) {
    if (strlen($state) != 2) {
        return 'America/Los_Angeles';
    }
    else {
        $state_timezones = array(
            'AK' => array(
                'name' => 'ALASKA',
                'timezone' => 'America/Anchorage',
                'timediff' => '-9',
            ),
            'AL' => array(
                'name' => 'ALABAMA',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'AR' => array(
                'name' => 'ARKANSAS',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'AZ' => array(
                'name' => 'ARIZONA',
                'timezone' => 'America/Denver',
                'timediff' => '-7',
            ),
            'CA' => array(
                'name' => 'CALIFORNIA',
                'timezone' => 'America/Los_Angeles',
                'timediff' => '-8',
            ),
            'CO' => array(
                'name' => 'COLORADO',
                'timezone' => 'America/Denver',
                'timediff' => '-7',
            ),
            'CT' => array(
                'name' => 'CONNECTICUT',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'DC' => array(
                'name' => 'DISTRICT OF COLUMBIA',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'DE' => array(
                'name' => 'DELAWARE',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'FL' => array(
                'name' => 'FLORIDA',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'GA' => array(
                'name' => 'GEORGIA',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'HI' => array(
                'name' => 'HAWAII',
                'timezone' => 'Pacific/Honolulu',
                'timediff' => '-10',
            ),
            'IA' => array(
                'name' => 'IOWA',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'ID' => array(
                'name' => 'IDAHO',
                'timezone' => 'America/Denver',
                'timediff' => '-7',
            ),
            'IL' => array(
                'name' => 'ILLINOIS',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'IN' => array(
                'name' => 'INDIANA',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'KS' => array(
                'name' => 'KANSAS',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'KY' => array(
                'name' => 'KENTUCKY',
                'timezone' => 'America/New_York',
                'timediff' => '-6',
            ),
            'LA' => array(
                'name' => 'LOUISIANA',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'MA' => array(
                'name' => 'MASSACHUSETTS',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'MD' => array(
                'name' => 'MARYLAND',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'ME' => array(
                'name' => 'MAINE',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'MI' => array(
                'name' => 'MICHIGAN',
                'timezone' => 'America/Detroit',
                'timediff' => '-5',
            ),
            'MN' => array(
                'name' => 'MINNESOTA',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'MO' => array(
                'name' => 'MISSOURI',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'MS' => array(
                'name' => 'MISSISSIPPI',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'MT' => array(
                'name' => 'MONTANA',
                'timezone' => 'America/Denver',
                'timediff' => '-7',
            ),
            'NC' => array(
                'name' => 'NORTH CAROLINA',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'ND' => array(
                'name' => 'NORTH DAKOTA',
                'timezone' => 'America/Denver',
                'timediff' => '-7',
            ),
            'NE' => array(
                'name' => 'NEBRASKA',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'NH' => array(
                'name' => 'NEW HAMPSHIRE',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'NJ' => array(
                'name' => 'NEW JERSEY',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'NM' => array(
                'name' => 'NEW MEXICO',
                'timezone' => 'America/Denver',
                'timediff' => '-7',
            ),
            'NV' => array(
                'name' => 'NEVADA',
                'timezone' => 'America/Los_Angeles',
                'timediff' => '-8',
            ),
            'NY' => array(
                'name' => 'NEW YORK',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'OH' => array(
                'name' => 'OHIO',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'OK' => array(
                'name' => 'OKLAHOMA',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'OR' => array(
                'name' => 'OREGON',
                'timezone' => 'America/Los_Angeles',
                'timediff' => '-8',
            ),
            'PA' => array(
                'name' => 'PENNSYLVANIA',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'RI' => array(
                'name' => 'RHODE ISLAND',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'SC' => array(
                'name' => 'SOUTH CAROLINA',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'SD' => array(
                'name' => 'SOUTH DAKOTA',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'TN' => array(
                'name' => 'TENNESSEE',
                'timezone' => 'America/Chicago',
                'timediff' => '-5',
            ),
            'TX' => array(
                'name' => 'TEXAS',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'UT' => array(
                'name' => 'UTAH',
                'timezone' => 'America/Denver',
                'timediff' => '-7',
            ),
            'VA' => array(
                'name' => 'VIRGINIA',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'VT' => array(
                'name' => 'VERMONT',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'WA' => array(
                'name' => 'WASHINGTON',
                'timezone' => 'America/Los_Angeles',
                'timediff' => '-8',
            ),
            'WI' => array(
                'name' => 'WISCONSIN',
                'timezone' => 'America/Chicago',
                'timediff' => '-6',
            ),
            'WV' => array(
                'name' => 'WEST VIRGINIA',
                'timezone' => 'America/New_York',
                'timediff' => '-5',
            ),
            'WY' => array(
                'name' => 'WYOMING',
                'timezone' => 'America/Denver',
                'timediff' => '-7',
            ),
        );  

        $timezone = $state_timezones[$state]['timezone'];

        if ($timezone > "") {
            return $timezone;
        }
        else {
            return 'America/Los_Angeles';
        }

        
    }
}