<?php 
/*
Plugin Name: CLICKS 3 Integration
Plugin URI: https://reachagency.co
description: Theme fields, styles, templates, etc. for CLICKS 3 theme.
Version: 1.1
Author: Reach Agency
License: GPL2
*/





/*** Making jQuery load from Google Library
----------------------------------------------------------------------------------------------------------------------------------*/
function clicks_use_jquery() {
	if (!is_admin()) {
		// comment out the next two lines to load the local copy of jQuery
		wp_deregister_script('jquery');
		wp_register_script('jquery', '//ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js', false, '1.11.3');
		wp_enqueue_script('jquery');
	}
}
add_action('init', 'clicks_use_jquery');





/*** Load Stylesheets and Scripts
--------------------------------------------------------------------------------------------------*/
function clicks_load_styles() {
  wp_enqueue_style( 'font-awesome', plugins_url( '/css/font-awesome.min.css' , __FILE__ ) );
  wp_enqueue_style( 'bootstrap', plugins_url( '/css/bootstrap.min.css' , __FILE__ ) );
  wp_enqueue_style( 'clicks', plugins_url( '/css/clicks.css' , __FILE__ ) );
}
add_action( 'wp_enqueue_scripts', 'clicks_load_styles', 9999 );

function clicks_load_scripts() {
  wp_register_script( 'bootstrap.min', plugins_url( '/js/bootstrap.min.js' , __FILE__ ), array(), '', true );
  wp_register_script( 'googlemap', 'https://maps.googleapis.com/maps/api/js?key=AIzaSyDyJBc4-iw83vEaS2sbF50F3XLRWU0gcqo&sensor=false',true);
  wp_register_script( 'jquery.cycle2.min', plugins_url( '/js/jquery.cycle2.min.js' , __FILE__ ), array(), '', true );
  wp_register_script( 'jquery.cycle2.carousel.min', plugins_url( '/js/jquery.cycle2.carousel.min.js' , __FILE__ ), array(), '', true );
  wp_register_script( 'jquery.cycle2.swipe.min', plugins_url( '/js/jquery.cycle2.swipe.min.js' , __FILE__ ), array(), '', true );
  wp_register_script( 'jquery.cycle2.center.min', plugins_url( '/js/jquery.cycle2.center.min.js' , __FILE__ ), array(), '', true );
  wp_register_script( 'jquery.fitvids', plugins_url( '/js/jquery.fitvids.js' , __FILE__ ), array(), '', false );

  wp_enqueue_script('bootstrap.min');
  wp_enqueue_script('googlemap');
  wp_enqueue_script('jquery.cycle2.min');
  wp_enqueue_script('jquery.cycle2.carousel.min');
  wp_enqueue_script('jquery.cycle2.swipe.min');
  wp_enqueue_script('jquery.cycle2.center.min');
  wp_enqueue_script('jquery.fitvids');
}
add_action( 'wp_enqueue_scripts', 'clicks_load_scripts' );





/*** ACF Google Maps
----------------------------------------------------------------------------------------------------------------------------------*/
function clicks_my_acf_init() {
	acf_update_setting('google_api_key', 'AIzaSyDyJBc4-iw83vEaS2sbF50F3XLRWU0gcqo');
}
add_action('acf/init', 'clicks_my_acf_init');





/*** CLICKS Colors
--------------------------------------------------------------------------------------------------*/
include( plugin_dir_path( __FILE__ ) . '/css/clicks-colors-css.php' );





/*** CLICKS Animations
--------------------------------------------------------------------------------------------------*/
include( plugin_dir_path( __FILE__ ) . '/css/clicks-animations-css.php' );





/*** Make CLICKS Pages Selectable as Homepage
----------------------------------------------------------------------------------------------------------------------------------*/
add_filter( 'get_pages', function ( $pages, $args )
{
    // First make sure this is an admin page, if not, bail
    if ( !is_admin() )
        return $pages;

    // Make sure that we are on the reading settings page, if not, bail
    global $pagenow;
    if ( 'options-reading.php' !== $pagenow )
        return $pages;

    // Remove the filter to avoid infinite loop
    remove_filter( current_filter(), __FUNCTION__ );

    $args = [
        'post_type'      => 'lp',
        'posts_per_page' => -1
    ];
    // Get the post type posts with get_posts to allow non hierarchical post types
    $new_pages = get_posts( $args );    

    /**
     * You need to decide if you want to add custom post type posts to the pages
     * already in the dropdown, or just want the custom post type posts in
     * the dropdown. I will handle both, just remove what is not needed
     */
    // If we only need custom post types
    $pages = $new_pages;

    // If we need to add custom post type posts to the pages
    // $pages = array_merge( $new_pages, $pages );

    return $pages;
}, 10, 2 );





/*** Create CLICKS Page Post Type
----------------------------------------------------------------------------------------------------------------------------------*/
function clicks_register_my_cpt_lp() {

	/**
	 * Post Type: CLICKS Pages.
	 */

	$labels = array(
		"name" => __( "CLICKS Pages", "custom-post-type" ),
		"singular_name" => __( "CLICKS Page", "custom-post-type" ),
	);

	$args = array(
		"label" => __( "CLICKS Pages", "custom-post-type" ),
		"labels" => $labels,
		"description" => "",
		"public" => true,
		"publicly_queryable" => true,
		"show_ui" => true,
		"delete_with_user" => false,
		"show_in_rest" => true,
		"rest_base" => "",
		"rest_controller_class" => "WP_REST_Posts_Controller",
		"has_archive" => false,
		"show_in_menu" => true,
		"show_in_nav_menus" => true,
		"exclude_from_search" => false,
		"capability_type" => "page",
		"map_meta_cap" => true,
		"hierarchical" => true,
    "rewrite" => array( "slug" => "lp", "with_front" => false ),
    //"rewrite" => false,
		"query_var" => true,
		"menu_position" => 1,
		"supports" => array( "title", "revisions", "author" ),
	);

	register_post_type( "lp", $args );
}

add_action( 'init', 'clicks_register_my_cpt_lp' );





/*** CLICKS Page Template
----------------------------------------------------------------------------------------------------------------------------------*/
add_filter('single_template', 'clicks_page_template');

function clicks_page_template($single) {

  global $post;

  /* Checks for single template by post type */
  if ( $post->post_type == 'lp' ) {
    if ( file_exists( plugin_dir_path( __FILE__ ) . '/clicks_page.php' ) ) {
      return plugin_dir_path( __FILE__ ) . '/clicks_page.php';
    }
  }

  return $single;

}





/*** Create Testimonial Post Type
----------------------------------------------------------------------------------------------------------------------------------*/
function clicks_register_my_cpt_testimonial() {

	/**
	 * Post Type: Testimonials.
	 */

	$labels = array(
		"name" => __( "Testimonials", "custom-post-type" ),
		"singular_name" => __( "Testimonial", "custom-post-type" ),
	);

	$args = array(
		"label" => __( "Testimonials", "custom-post-type-ui" ),
		"labels" => $labels,
		"description" => "",
		"public" => true,
		"publicly_queryable" => true,
		"show_ui" => true,
		"delete_with_user" => false,
		"show_in_rest" => false,
		"rest_base" => "",
		"rest_controller_class" => "WP_REST_Posts_Controller",
		"has_archive" => false,
		"show_in_menu" => true,
		"show_in_nav_menus" => true,
		"exclude_from_search" => false,
		"capability_type" => "post",
		"map_meta_cap" => true,
		"hierarchical" => false,
		"rewrite" => array( "slug" => "testimonial", "with_front" => true ),
		"query_var" => true,
		"menu_position" => 21,
		"supports" => array( "title", "revisions", "author", "page-attributes" ),
		"taxonomies" => array( "category" ),
	);

	register_post_type( "testimonial", $args );
}

add_action( 'init', 'clicks_register_my_cpt_testimonial' );





/*** Create FAQ Post Type
----------------------------------------------------------------------------------------------------------------------------------*/
function clicks_register_my_cpt_faq() {

	/**
	 * Post Type: FAQs.
	 */

	$labels = array(
		"name" => __( "FAQs", "custom-post-type" ),
		"singular_name" => __( "FAQ", "custom-post-type" ),
	);

	$args = array(
		"label" => __( "FAQs", "custom-post-type" ),
		"labels" => $labels,
		"description" => "",
		"public" => true,
		"publicly_queryable" => true,
		"show_ui" => true,
		"delete_with_user" => false,
		"show_in_rest" => false,
		"rest_base" => "",
		"rest_controller_class" => "WP_REST_Posts_Controller",
		"has_archive" => false,
		"show_in_menu" => true,
		"show_in_nav_menus" => true,
		"exclude_from_search" => false,
		"capability_type" => "post",
		"map_meta_cap" => true,
		"hierarchical" => false,
		"rewrite" => array( "slug" => "faq", "with_front" => true ),
    "query_var" => true,
    "menu_position" => 21,
		"supports" => array( "title", "editor", "thumbnail" ),
		"taxonomies" => array( "category" ),
	);

	register_post_type( "faq", $args );
}

add_action( 'init', 'clicks_register_my_cpt_faq' );





/*** Hex to RBG Conversion
----------------------------------------------------------------------------------------------------------------------------------*/
function hex2rgb($hex) {
  $hex = str_replace("#", "", $hex);

  if(strlen($hex) == 3) {
    $r = hexdec(substr($hex,0,1).substr($hex,0,1));
    $g = hexdec(substr($hex,1,1).substr($hex,1,1));
    $b = hexdec(substr($hex,2,1).substr($hex,2,1));
  } else {
    $r = hexdec(substr($hex,0,2));
    $g = hexdec(substr($hex,2,2));
    $b = hexdec(substr($hex,4,2));
  }
  $rgb = array($r, $g, $b);

  return $rgb; // returns an array with the rgb values
}





/*** Field Groups
----------------------------------------------------------------------------------------------------------------------------------*/

// Panel Fields
include( plugin_dir_path( __FILE__ ) . 'fields/hero-fields.php' );

include( plugin_dir_path( __FILE__ ) . 'fields/content-fields.php' );

include( plugin_dir_path( __FILE__ ) . 'fields/footer-fields.php' );


// Global Settings Fields
include( plugin_dir_path( __FILE__ ) . 'fields/testimonial-fields.php' );

include( plugin_dir_path( __FILE__ ) . 'fields/logo-fields.php' );

include( plugin_dir_path( __FILE__ ) . 'fields/contact-fields.php' );

include( plugin_dir_path( __FILE__ ) . 'fields/tracking-fields.php' );


// Style Settings Fields
include( plugin_dir_path( __FILE__ ) . 'fields/font-fields.php' );

include( plugin_dir_path( __FILE__ ) . 'fields/color-fields.php' );

include( plugin_dir_path( __FILE__ ) . 'fields/animation-fields.php' );


// Page Fields
include( plugin_dir_path( __FILE__ ) . 'fields/page-tracking-fields.php' );

include( plugin_dir_path( __FILE__ ) . 'fields/page-description-fields.php' );






/*** CLICKS Shortcodes
--------------------------------------------------------------------------------------------------*/
include( plugin_dir_path( __FILE__ ) . 'shortcodes/clicks_shortcodes.php' );





/*** ACF 'Global Settings' Options Pages
----------------------------------------------------------------------------------------------------------------------------------*/
if( function_exists('acf_add_options_page') ) {
  
  // Global Settings
	acf_add_options_page(array(
		'menu_title'	=> 'Global Settings',
		'menu_slug' 	=> 'global-settings',
		'redirect' => true,
		'position' => 1
  ));
	
	acf_add_options_sub_page(array(
		'page_title' 	=> 'Logos',
		'menu_title'	=> 'Logos',
		'parent_slug'	=> 'global-settings',
  ));

  acf_add_options_sub_page(array(
		'page_title' 	=> 'Contact Info',
		'menu_title'	=> 'Contact Info',
		'parent_slug'	=> 'global-settings',
  ));

  acf_add_options_sub_page(array(
		'page_title' 	=> 'Tracking Info',
		'menu_title'	=> 'Tracking Info',
		'parent_slug'	=> 'global-settings',
  ));



  // Style Settings
  acf_add_options_page(array(
		'menu_title'	=> 'Style Settings',
		'menu_slug' 	=> 'style-settings',
		'redirect' => true,
		'position' => 1
  ));
	
	acf_add_options_sub_page(array(
		'page_title' 	=> 'Fonts',
		'menu_title'	=> 'Fonts',
		'parent_slug'	=> 'style-settings',
  ));

  acf_add_options_sub_page(array(
		'page_title' 	=> 'Colors',
		'menu_title'	=> 'Colors',
		'parent_slug'	=> 'style-settings',
  ));

  acf_add_options_sub_page(array(
		'page_title' 	=> 'Animations',
		'menu_title'	=> 'Animations',
		'parent_slug'	=> 'style-settings',
  ));
	
}





/*** Custom Columns
----------------------------------------------------------------------------------------------------------------------------------*/
function pagecolumns_header($columns) {
	$columns = array(
		'cb'	 	=> '<input type="checkbox" />',
		'title' 	=> 'Page Title',
		'description'  =>  'Description',
		'referral' => 'Referral ID',
		'3wp_broadcast' => 'Broadcasted',
		'author' => 'Author',
		'date'		=>	'Date',
	);
	return $columns;
}

function pagecolumns_content($column) {
	global $post;
  if($column == 'description') {
		echo get_field('page_description', $post->ID);
  }
  if($column == 'referral') {
    if( get_field('page_cr_referral_id', $postID) ) {
      echo get_field('page_cr_referral_id', $postID);
    } elseif( get_field('global_cr_referral_id','option') ) {
      echo get_field('global_cr_referral_id','option');
    }
  }
}

add_action("manage_pages_custom_column", "pagecolumns_content");
add_filter("manage_edit-lp_columns", "pagecolumns_header");

function pagecolumns_sortable( $columns ) {
  $columns['description'] = 'description';
  $columns['referral'] = 'referral';
  $columns['date'] = 'date';
	return $columns;
}

add_filter("manage_edit-lp_sortable_columns", "pagecolumns_sortable" );

?>