<?php

namespace threewp_broadcast\premium_pack\bulk_cloner\components;

/**
	@brief		Allow setting of the statuses.
	@since		2021-06-14 10:00:40
**/
class Statuses
	extends \threewp_broadcast\premium_pack\base
{
	/**
		@brief		This is the full option key that is the data_type and the option_key.
		@since		2018-02-21 20:28:39
	**/
	public static $data_type_keys = [
		'blog_status_archived' => 'Archived',
		'blog_status_mature' => 'Mature',
		'blog_status_spam' => 'Spam',
		'blog_status_deleted' => 'Deleted',
	];

	public function _construct()
	{
		$this->add_action( 'broadcast_bulk_cloner_display_export_options' );
		$this->add_action( 'broadcast_bulk_cloner_generate_blog_state' );
		$this->add_action( 'broadcast_bulk_cloner_update_blog' );
	}

	/**
		@brief		Show our option(s).
		@since		2018-02-21 19:31:38
	**/
	public function broadcast_bulk_cloner_display_export_options( $action )
	{
		$og = $action->select->optgroup( 'optgroup_blog' );

		foreach( static::$data_type_keys as $key => $description )
			$og->option( 'Blog status - ' . $description, $key );
	}

	/**
		@brief		Add our data to the state.
		@since		2018-02-21 19:39:14
	**/
	public function broadcast_bulk_cloner_generate_blog_state( $action )
	{
		$bs = $action->blog_state;
		$export_options = broadcast_bulk_cloner()->get_export_options();
		$values = $export_options->get_export_options_select_value();

		foreach( static::$data_type_keys as $data_type_key => $description )
		{
			// Is our data type and key in the values to export?
			if ( in_array( $data_type_key, $values ) )
			{
				// Create a form input that helps things like the Manual Cloner to present a nicer UI for the user.
				$bs->form()->checkbox( $data_type_key )
					->label( $description );

				// Tell the blog state that when importing, it can expect the combo of broadcast and redirect_visitors_url.
				$bs->expect_key( 'blog', $data_type_key );
				// Retrieve the current redirect option from this blog.
				$key = str_replace( 'blog_status_', '', $data_type_key );
				$value = get_blog_status( get_current_blog_id(), $key );
				// And put the value into the blog state, ready to be viewed or changed.
				$bs->set_data( 'blog', $data_type_key, $value );
			}
		}
	}

	/**
		@brief		Update the blog with our options.
		@since		2018-02-21 19:31:48
	**/
	public function broadcast_bulk_cloner_update_blog( $action )
	{
		$bs = $action->blog_state;		// Conv
		$blog_id = $bs->get_blog_id();	// Conv.

		foreach( static::$data_type_keys as $data_type_key => $description )
		{
			if ( ! $bs->collection( 'blog' )->has( $data_type_key ) )
				continue;
			$new_value = $bs->collection( 'blog' )->get( $data_type_key );
			$key = str_replace( 'blog_status_', '', $data_type_key );
			$this->debug( "Setting new blog status %s to %s", $key, $new_value );
			update_blog_status( get_current_blog_id(), $key, $new_value );
		}
	}
}
