<?php

namespace threewp_broadcast\premium_pack\bulk_cloner\spreadsheet;

/**
	@brief		Container for all the data that needs to be passed back and forth during partial exporting.
	@since		2022-04-03 16:16:14
**/
class Partial_Container
	extends \threewp_broadcast\collection
{
	/**
		@brief		Count how many blogs it has left to generate.
		@since		2022-04-03 17:57:24
	**/
	public function count_blogs_to_generate()
	{
		return count( $this->get( 'blogs_to_generate' ) );
	}

	/**
		@brief		Return the blog states have been generated since last time.
		@see		remember_blog_states()
		@since		2022-04-03 17:44:19
	**/
	public function diff_generated_blog_states()
	{
		$old_keys = $this->get( 'generated_blog_states', [] );
		$new_keys = array_keys( $this->get( 'blog_states' )->to_array() );
		$diff = array_diff( $new_keys, $old_keys );
		$r = new static();
		foreach( $diff as $key )
			$r->set( $key, $this->get( 'blog_states' )->get( $key ) );
		return $r;
	}

	/**
		@brief		Return whether there are more blogs to generate.
		@since		2022-04-03 16:16:53
	**/
	public function has_blogs_to_generate()
	{
		return $this->count_blogs_to_generate() > 0;
	}

	/**
		@brief		Pop the next site to generate.
		@since		2022-04-03 16:15:28
	**/
	public function pop_blog_to_generate()
	{
		$array = $this->get( 'blogs_to_generate' );
		$next = array_shift( $array );
		$this->set( 'blogs_to_generate', $array );
		return $next;
	}

	/**
		@brief		Remember which blog states have been generated so far.
		@since		2022-04-03 17:42:43
	**/
	public function remember_blog_states()
	{
		$keys = array_keys( $this->get( 'blog_states' )->to_array() );
		$this->set( 'generated_blog_states', $keys );
	}
}