<?php

namespace threewp_broadcast\premium_pack\code_snippets;

use ParseError;

/**
	@brief		Individual snippet.
	@since		2021-09-29 20:30:03
**/
class Snippet
	extends \threewp_broadcast\collection
{
	/**
		@brief		Create a new snippet.
		@since		2021-09-29 21:41:27
	**/
	public static function create()
	{
		$r = new static();
		$id = time() . rand( 1000, 9999 );
		$r->set( 'id', $id );
		$r->set( 'description', 'Snippet ' . $id );
		return $r;
	}

	/**
		@brief		Get the blog IDs this snippet is enabled on.
		@since		2021-09-29 21:47:08
	**/
	public function get_blogs()
	{
		return $this->collection( 'blogs' );
	}

	/**
		@brief		Is this snippet active on this blog?
		@since		2021-09-29 20:34:10
	**/
	public function is_active_here()
	{
		return $this->collection( 'blogs' )->has( get_current_blog_id() );
	}

	/**
		@brief		Is this snippet enabled?
		@since		2021-09-29 21:44:28
	**/
	public function is_enabled()
	{
		return $this->get( 'enabled', false );
	}

	/**
		@brief		Run this snippet's code.
		@since		2021-09-29 22:12:38
	**/
	public function run()
	{
		try
		{
			$code = $this->get( 'code' );
			eval( $code );
		}
		catch ( ParseError $pe )
		{
			echo sprintf( 'Broadcast Code Snippet PHP error: %s', $pe->getMessage() );
		}
	}

	/**
		@brief		Set which blogs this snippet is active on.
		@since		2021-09-29 21:32:07
	**/
	public function set_blogs( $blog_ids )
	{
		$blogs = $this->collection( 'blogs' );
		$blogs->flush();
		foreach( $blog_ids as $blog_id )
			$blogs->set( $blog_id, $blog_id );
		return $this;
	}

	/**
		@brief		Set the enabled status of the snippet.
		@since		2021-09-29 21:44:52
	**/
	public function set_enabled( $enabled )
	{
		$this->set( 'enabled', $enabled );
		return $this;
	}
}
